@@ -8,18 +8,27 @@ use warnings;
 use Moose;
 use App::backimap::Status;
 use App::backimap::Status::Folder;
+use App::backimap::IMAP;
+use App::backimap::Storage;
 
 has status => (
     is => 'rw',
     isa => 'App::backimap::Status',
 );
 
+has imap => (
+    is => 'rw',
+    isa => 'App::backimap::IMAP',
+);
+
+has storage => (
+    is => 'rw',
+    isa => 'App::backimap::Storage',
+);
+
 use Getopt::Long         qw( GetOptionsFromArray );
 use Pod::Usage;
 use URI;
-use App::backimap::Utils qw( imap_uri_split );
-use IO::Prompt           qw( prompt );
-use Mail::IMAPClient;
 use File::Spec::Functions qw( catfile );
 use File::Path            qw( mkpath );
 use Git::Wrapper;
@@ -54,74 +63,35 @@ sub new {
 }
 
 
-sub config {
+sub setup {
     my ( $self, $str ) = @_;
 
     my $uri = URI->new($str);
-    my $imap_cfg = imap_uri_split($uri);
 
-    $imap_cfg->{'password'} = prompt('Password: ', -te => '*' )
-        unless defined $imap_cfg->{'password'};
+    $self->imap(
+        App::backimap::IMAP->new( uri => $uri )
+    );
 
     $self->status(
         App::backimap::Status->new(
-            server    => $imap_cfg->{'host'},
-            user      => $imap_cfg->{'user'},
+            server    => $self->imap->host,
+            user      => $self->imap->user,
         )
     );
 
-    $self->{'config'} = $imap_cfg;
-}
-
-
-sub login {
-    my ($self) = @_;
-
-    # make sure we can make a secure connection
-    require IO::Socket::SSL
-        if $self->{'config'}{'secure'};
-
-    $self->{'imap'} = Mail::IMAPClient->new(
-        Server   => $self->{'config'}{'host'},
-        Port     => $self->{'config'}{'port'},
-        Ssl      => $self->{'config'}{'secure'},
-        User     => $self->{'config'}{'user'},
-        Password => $self->{'config'}{'password'},
-
-        # enable imap uid per folder
-        Uid => 1,
-    )
-        or die "cannot establish connection: $@\n";
-}
-
-
-sub logout {
-    my ($self) = @_;
-
-    my $imap = $self->{'imap'};
-    $imap->logout()
-        if defined $imap && $imap->isa('Mail::IMAPClient');
-}
-
-
-sub setup {
-    my ($self) = @_;
-
     my $dir = $self->{'dir'};
     my $filename = catfile( $dir, "backimap.json" );
-    my $git = Git::Wrapper->new($dir);
-    $self->{'git'} = $git;
 
-    if ( $self->{'init'} ) {
-        die "directory $dir already initialized\n"
-            if -f $filename || -d catfile( $dir, ".git" );
+    $self->storage(
+        App::backimap::Storage->new(
+            dir => $dir,
+            init => $self->{'init'},
+        ),
+    );
 
-        mkpath($dir);
-        $git->init();
-
-        # save initial status in the Git repository
-        $self->save();
-    }
+    # save initial status
+    $self->save()
+        if $self->{'init'};
 
     my $status = App::backimap::Status->load($filename);
 
@@ -138,34 +108,21 @@ sub save {
     my ($self) = @_;
 
     my $git = $self->{'git'};
-    croak "must setup git repo first"
-        unless defined $git && $git->isa('Git::Wrapper');
 
     croak "must define status first"
         unless defined $self->status;
 
-    my $dir = $self->{'dir'};
-    my $filename = catfile( $dir, "backimap.json" );
-
-    $self->status->store($filename);
-
-    $git->add($filename);
-    $git->commit( { message => "save status" }, $filename );
+    $self->status->store( catfile( $self->{'dir'}, "backimap.json" ) );
+    $self->storage->put("save status");
 }
 
 
 sub backup {
     my ($self) = @_;
 
-    my $git = $self->{'git'};
-    croak "must init git repo first"
-        unless defined $git && $git->isa('Git::Wrapper');
+    my $imap = $self->imap->client;
 
-    my $imap = $self->{'imap'};
-    croak "imap connection unavailable"
-        unless defined $imap && $imap->isa('Mail::IMAPClient');
-
-    my $path = $self->{'config'}{'path'};
+    my $path = $self->imap->path;
     $path =~ s#^/+##;
 
     my @folder_list = $path ne '' ? $path : $imap->folders;
@@ -197,27 +154,15 @@ sub backup {
         print STDERR " * $folder ($unseen/$count)"
             if $self->{'verbose'};
 
-        my $local_folder = catfile( $self->{'dir'}, $folder );
-        mkpath( $local_folder );
-        chdir $local_folder;
-
         $imap->examine($folder);
         for my $msg ( $imap->messages ) {
-            next if -f $msg;
+            my $file = catfile( $folder, $msg );
+            next if $self->storage->get($file);
 
             my $fetch = $imap->fetch( $msg, 'RFC822' );
-
-            open my $file, ">", $msg
-                or die "message $msg: $!";
-
-            print $file $fetch->[2];
-            close $file;
-
-            $git->add( catfile( $local_folder, $msg ) );
+            $self->storage->put( "save message $file", $file => $fetch->[2] );
         }
 
-        eval { $git->commit({ all => 1, message => "save messages from $folder" }) };
-
         print STDERR "\n"
             if $self->{'verbose'};
     }
@@ -230,11 +175,8 @@ sub run {
     my @args = @{ $self->{'args'} };
     $self->usage unless @args == 1;
 
-    $self->config(@args);
-    $self->setup();
-    $self->login();
+    $self->setup(@args);
     $self->backup();
-    $self->logout();
     $self->save();
 
     my $spent = ( time - $^T ) / 60;
@@ -260,7 +202,7 @@ App::backimap - backups imap mail
 
 =head1 VERSION
 
-version 0.00_05
+version 0.00_06
 
 =head1 SYNOPSIS
 
@@ -273,21 +215,11 @@ version 0.00_05
 
 Creates a new program instance with command line arguments.
 
-=head2 config
-
-Setups configuration and prompts for password if needed.
-
-=head2 login
-
-Connects to IMAP server.
-
-=head2 logout
-
-Disconnects from IMAP server.
-
 =head2 setup
 
-Open Git repository (initialize if asked) and load previous status.
+Setups configuration and prompts for password if needed.
+Then opens Git repository (initialize if asked) and
+load previous status.
 
 =head2 save
 
